package com.huatek.busi.entity;

import java.io.Serializable;
import java.time.LocalDateTime;

import org.springframework.format.annotation.DateTimeFormat;

import com.huatek.torch.sql.annotation.FieldName;
import com.huatek.torch.sql.annotation.TableName;
import com.huatek.torch.sql.beans.Pojo;

import lombok.Data;
import lombok.EqualsAndHashCode;

/**
 * 设备预警表 entity
 *
 * @author anker_cheng
 * @date Jul 18, 2019 2:45:23 PM
 */
@Data
@EqualsAndHashCode(callSuper = false)
@TableName("busi_eqm_warning")
public class BusiEqmWarning extends Pojo implements Serializable  {
    private static final long serialVersionUID = 1L;

    /**
     * 主键ID
     */
    @FieldName("id")
    private String id;
 
    /**
     * 告警单编号
     */
    @FieldName("warning_code")
    private String warningCode;
    
    /**
     * 告警标准（1告警2、预警），默认为1
     */
    @FieldName("warning_flag")
    private String warningFlag;

    /**
     * 故障大类1：变压器，2：gis，3：避雷器，4：测温柜
     */
    @FieldName("eq_type")
    private String eqType;

    /**
     * 故障小类 
			1—油色谱  
			2—局放 
			3—铁芯接地
			4-红外测温
			5-SF6告警
			6-避雷器告警
			7-测温柜测温告警
			8-铁芯夹件告警 
			9-运行工况告警
			10-测温点增长率
			11-读表数据告警 
			12-容性告警
			13-套管告警 
     */
    @FieldName("gz_type")
    private String gzType;

    /**
     * 预警类别编码
     */
    @FieldName("warning_type_code")
    private String warningTypeCode;

    /**
     * 预警信息
     */
    @FieldName("warning_msg")
    private String warningMsg;

    /**
     * 预警时间
     */
    @FieldName("warning_time")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime warningTime;

    /**
     * 创建时间
     */
    @FieldName("create_time")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime createTime;

    /**
     * 删除标识 0:未处理 1：已处理
     */
    @FieldName("del_flag")
    private String delFlag;

    /**
     * 处理状态:0——未处理、1——已处理
     */
    @FieldName("deal_status")
    private String dealStatus;

    /**
     * 处理结果
     */
    @FieldName("deal_msg")
    private String dealMsg;

    /**
     * 故障类型 1：故障 2：误报
     */
    @FieldName("warning_type")
    private String warningType;

    /**
     * 站点Id
     */
    @FieldName("site_id")
    private String siteId;

    /**
     * 传感器id
     */
    @FieldName("sensor_id")
    private String sensorId;

    /**
     * 处理时间
     */
    @FieldName("processing_time")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime processingTime;

    /**
     * 处理人
     */
    @FieldName("processing_user")
    private String processingUser;

    /**
     * 外挂设备安装位置
     */
    @FieldName("sub_equipment_install_location")
    private String subEquipmentInstallLocation;

    /**
     * 外挂设标牌
     */
    @FieldName("sub_equipment_brand")
    private String subEquipmentBrand;

    /**
     * 外挂设备生产厂家
     */
    @FieldName("sub_equipment_manufactor")
    private String subEquipmentManufactor;

    /**
     * 传感器安装位置
     */
    @FieldName("sensor_install_location")
    private String sensorInstallLocation;

    /**
     * 传感器标牌
     */
    @FieldName("sensor_brand")
    private String sensorBrand;

    /**
     * 传感器生产厂家
     */
    @FieldName("sensor_manufactor")
    private String sensorManufactor;

    /**
     * 外挂设备名称
     */
    @FieldName("sub_equipment_name")
    private String subEquipmentName;

    /**
     * 传感器名称
     */
    @FieldName("sensor_name")
    private String sensorName;

	/** @Fields subTreeId : 外挂设备树id */
    @FieldName("sub_tree_id")
	private String subTreeId;
	
	/** @Fields mainTreeId : 站点树ID */
    @FieldName("main_tree_id")
	private String mainTreeId;

    /**
     * 主设备ID
     */
    @FieldName("main_equipment_id")
    private String mainEquipmentId;

    /**
     * 外挂设备ID
     */
    @FieldName("sub_equipment_id")
    private String subEquipmentId;

    /**
     * 设备状态
     */
    @FieldName("run_state")
    private String runState;
}