package com.huatek.busi.entity;

import java.io.Serializable;
import java.math.BigDecimal;
import java.time.LocalDateTime;

import org.springframework.format.annotation.DateTimeFormat;

import com.huatek.torch.sql.annotation.FieldName;
import com.huatek.torch.sql.annotation.TableName;
import com.huatek.torch.sql.beans.Pojo;

import lombok.Data;
import lombok.EqualsAndHashCode;

/**
 * 断路器/GIS——SF6气体压力：024004、024005 entity
 *
 * @author anker_cheng
 * @date Aug 2, 2019 2:12:59 PM
 */
@Data
@EqualsAndHashCode(callSuper = false)
@TableName("gis_sf6_pressure")
public class GisSf6Pressure extends Pojo implements Serializable  {

    private static final long serialVersionUID = 1L;

    /**
     * 主键
     */
    @FieldName("id")
    private String id;

    /**
     * CAC 的唯一标识（17 位码）
     */
    @FieldName("cac_id")
    private String cacId;

    /**
     * 监测数据组的数目
     */
    @FieldName("data_node_num")
    private BigDecimal dataNodeNum;

    /**
     * 在线监测装置的唯一标识（17 位码）
     */
    @FieldName("sensor_id")
    private String sensorId;

    /**
     * 监测类型编码
     */
    @FieldName("cac_monitor_type_code")
    private String cacMonitorTypeCode;

    /**
     * 在线监测装置对应的被监测设备的 唯一标识（17 位码）
     */
    @FieldName("equipment_id")
    private String equipmentId;

    /**
     * 数据采集时间，格式为：yyyy-MM-dd HH:mm:ss
     */
    @FieldName("time_stamp")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime timeStamp;

    /**
     * 主设备编号
     */
    @FieldName("equipment_code")
    private String equipmentCode;

    /**
     * 外挂设备编号
     */
    @FieldName("ext_equipment_code")
    private String extEquipmentCode;

    /**
     * 站点ID
     */
    @FieldName("site_id")
    private String siteId;

    /**
     * 被监测设备相别
     */
    @FieldName("phase")
    private String phase;

    /**
     * 温度（℃）
     */
    @FieldName("temperature")
    private String temperature;

    /**
     * 绝对压力 (MPa)
     */
    @FieldName("absolute_pressure")
    private String absolutePressure;

    /**
     * 密度(kg/m3)
     */
    @FieldName("density")
    private String density;

    /**
     * 微水
     */
    @FieldName("moisture")
    private String moisture;

    /**
     * 压力（20℃）
     */
    @FieldName("pressure20C")
    private String pressure20c;

    /**
     * 主设备ID
     */
    @FieldName("main_equipment_id")
    private String mainEquipmentId;

    /**
     * 外挂设备ID
     */
    @FieldName("sub_equipment_id")
    private String subEquipmentId;

    /**
     * 站点设备树ID
     */
    @FieldName("main_tree_id")
    private String mainTreeId;

    /**
     * 外挂设备树ID
     */
    @FieldName("sub_tree_id")
    private String subTreeId;
    
    /**
     * 装置灵敏度
     */
    @FieldName("equipment_sensitivity")
    private String equipmentSensitivity;
    
    /**
     * 日泄露率
     */
    @FieldName("daily_leak_rate")
    private String dailyLeakLate;
   
    /**
     * hbase中的rowkey
     */
    @FieldName("hbase_rowkey")
    private String hbaseRowkey;
    
    /**
           * 折合到20度的压力值
     */
    @FieldName("pressure")
    private String pressure;
    
     /**
      * 1:有效  2：无效
      */
    @FieldName("valided")
    private String valided;
}