package com.huatek.busi.entity;

import java.io.Serializable;
import java.math.BigDecimal;
import java.time.LocalDateTime;

import org.springframework.format.annotation.DateTimeFormat;

import com.huatek.torch.sql.annotation.FieldName;
import com.huatek.torch.sql.annotation.TableName;
import com.huatek.torch.sql.beans.Pojo;

import lombok.Data;
import lombok.EqualsAndHashCode;

/**
 * 变压器/电抗器——油中溶解气体：021002 entity
 *
 * @author anker_cheng
 * @date Sep 26, 2019 2:50:45 PM
 */
@Data
@EqualsAndHashCode(callSuper = false)
@TableName("transformer_epa_data")
public class TransformerEpaData extends Pojo implements Serializable  {

    private static final long serialVersionUID = 1L;

    /**
     * 主键
     */
    @FieldName("id")
    private String id;

    /**
     * CAC 的唯一标识（17 位码）
     */
    @FieldName("cac_id")
    private String cacId;

    /**
     * 监测数据组的数目
     */
    @FieldName("data_node_num")
    private BigDecimal dataNodeNum;

    /**
     * 在线监测装置的唯一标识（17 位码）
     */
    @FieldName("sensor_id")
    private String sensorId;

    /**
     * 监测类型编码
     */
    @FieldName("cac_monitor_type_code")
    private String cacMonitorTypeCode;

    /**
     * 在线监测装置对应的被监测设备的 唯一标识（17 位码）
     */
    @FieldName("equipment_id")
    private String equipmentId;

    /**
     * 数据采集时间，格式为：yyyy-MM-dd HH:mm:ss
     */
    @FieldName("time_stamp")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime timeStamp;

    /**
     * 主设备编号
     */
    @FieldName("equipment_code")
    private String equipmentCode;

    /**
     * 外挂设备编号
     */
    @FieldName("ext_equipment_code")
    private String extEquipmentCode;

    /**
     * 站点ID
     */
    @FieldName("site_id")
    private String siteId;

    /**
     * 被监测设备相别
     */
    @FieldName("phase")
    private String phase;

    /**
     * 氢气（μL/L）
     */
    @FieldName("h2")
    private String h2;

    /**
     * 甲烷（μL/L）
     */
    @FieldName("ch4")
    private String ch4;

    /**
     * 乙烷（μL/L）
     */
    @FieldName("c2h6")
    private String c2h6;

    /**
     * 乙烯（μL/L）
     */
    @FieldName("c2h4")
    private String c2h4;

    /**
     * 乙炔（μL/L）
     */
    @FieldName("c2h2")
    private String c2h2;

    /**
     * 一氧化碳（μL/L）
     */
    @FieldName("co")
    private String co;

    /**
     * 二氧化碳（μL/L）
     */
    @FieldName("co2")
    private String co2;

    /**
     * 氧气（μL/L）
     */
    @FieldName("o2")
    private String o2;

    /**
     * 氮气（μL/L）
     */
    @FieldName("n2")
    private String n2;

    /**
     * 温度（℃）
     */
    @FieldName("tmp")
    private String temperature;

    /**
     * 湿度（%）
     */
    @FieldName("dew")
    private String humidity;

    /**
     * 微水（μL/L）
     */
    @FieldName("mst")
    private String microMoisture;

    /**
     * 瓦斯继电器中故障气（μL/L）
     */
    @FieldName("flatgas")
    private String flatGas;

    /**
     * 总可燃气体（μL/L）
     */
    @FieldName("cmbugas")
    private String cmbuGas;

    /**
     * 总烃（μL/L）
     */
    @FieldName("total_hydrocarbon")
    private String totalHydrocarbon;

    /**
     * 载气压力(KPa)
     */
    @FieldName("gas_press")
    private String gasPress;

    /**
     * 水分
     */
    @FieldName("h2o")
    private String h2o;

    /**
     * 主设备ID
     */
    @FieldName("main_equipment_id")
    private String mainEquipmentId;

    /**
     * 外挂设备ID
     */
    @FieldName("sub_equipment_id")
    private String subEquipmentId;

    /**
     * 站点设备树ID
     */
    @FieldName("main_tree_id")
    private String mainTreeId;

    /**
     * 外挂设备树ID
     */
    @FieldName("sub_tree_id")
    private String subTreeId;
    
    /**
     * hbase中的rowkey
     */
    @FieldName("hbase_rowkey")
    private String hbaseRowkey;
}