package com.huatek.busi.vo;

import java.io.Serializable;
import java.time.LocalDate;
import java.time.LocalDateTime;

import org.springframework.format.annotation.DateTimeFormat;

import com.huatek.busi.common.annotation.ExcelAnnotation;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;

/**
 * 考评指标配置 Vo
 *
 * @author anker_cheng
 * @date Jul 23, 2019 2:06:26 PM
 */
@ApiModel(value = "BusiEvaMonitorConfigVo", description = "考评指标配置 Vo")
@Data
@EqualsAndHashCode(callSuper = false)
public class BusiEvaMonitorConfigVo implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 主键ID
     */
    @ApiModelProperty(name = "id", value = "主键ID")
    private String id;

    /**
     * 监测设备(类型ID)
     */
    @ApiModelProperty(name = "monitorTypeId", value = "监测设备(类型ID)")
    private String monitorTypeId;
    
    @ExcelAnnotation(exportName = "被监测设备")
    private String monitorTypeName;

    /**
     * 监测单元
     */
    @ApiModelProperty(name = "monitorUnitId", value = "在线监测单元")
    private String monitorUnitId;
    @ExcelAnnotation(exportName = "监测单元")
    private String monitorUnitName;

    /**
     * sensor id
     */
    @ApiModelProperty(name = "sensorId", value = "sensor编码")
    @ExcelAnnotation(exportName = "传感器编码")
    private String sensorId;
    
    /**
     * 传感器名称
     */
    @ApiModelProperty(name = "sensorName", value = "传感器名称")
    @ExcelAnnotation(exportName = "传感器名称")
    private String sensorName;
    
    /**
     * 标牌
     */
    @ApiModelProperty(name = "brandName", value = "标牌")
    @ExcelAnnotation(exportName = "标牌")
    private String brandName;
    
    /**
     * 安装位置
     */
    @ApiModelProperty(name = "installLocation", value = "安装位置")
    @ExcelAnnotation(exportName = "安装位置")
    private String installLocation;
    
    /**
     * 生产厂家
     */
    @ApiModelProperty(value = "生产厂家", name = "manufactor")
    private String manufactor;
    
    
    /**
     * 传感器投运日期
     */
    @ApiModelProperty(name = "useDate", value = "投运日期")
    @ExcelAnnotation(exportName = "投运日期")
    private String useDate;
    
    /**
     * 单元考核指标
     */
    @ApiModelProperty(name = "indicatorsId", value = "单元考核指标")
    private String indicatorsId;
    
    /**
     * 单元考核指标名称
     */
    @ApiModelProperty(name = "indicatorsName", value = "单元考核指标名称")
    @ExcelAnnotation(exportName = "单元考核指标")
    private String indicatorsName;



    /**
     * 采集频率:d——每天；m——每月；y——每年；hy——每半年
     */
    @ApiModelProperty(name = "sampleFrequency", value = "采集频率:d——每天；m——每月；y——每年；hy——每半年")
    private String sampleFrequency;
    
    /**
     * 考核频率从
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    private LocalDate evaFrom;

    /**
     * 考核频率至
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    private LocalDate evaTo;
    
    /**
     * 导出考核频率
     */
    private String exportEva;

    /**
     * 备注
     */
    @ApiModelProperty(name = "mark", value = "备注")
    private String mark;
    

    @ApiModelProperty(name = "creatorName", value = "创建人名称")
    @ExcelAnnotation(exportName = "创建人")
    private String creatorName;
    
    /**
     * 创建时间
     */
    @ApiModelProperty(name = "createdTime", value = "创建时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @ExcelAnnotation(exportName = "创建时间")
    private LocalDateTime createdTime;
    
    @ApiModelProperty(name = "modifierName", value = "修改人名称")
    private String modifierName;

    /**
     * 修改时间
     */
    @ApiModelProperty(name = "modifiedTime", value = "修改时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime modifiedTime;
    
    /**
     * 说明:告警
     */
    @ApiModelProperty(name = "description", value = "说明:告警")
    private String description;

    /**
     * 状态：1——可用；2——不可用
     */
    @ApiModelProperty(name = "status", value = "状态：1——可用；2——不可用")
    private String status;

    /**
     * 创建人ID
     */
    @ApiModelProperty(name = "creator", value = "创建人ID")
    private String creator;


    /**
     * 修改人
     */
    @ApiModelProperty(name = "modifier", value = "修改人")
    private String modifier;


    @ApiModelProperty(name = "type", value = "大类;1：变压器   2:GIS  3:测温   4：避雷器")
    private String type;

    @ApiModelProperty(name = "equipmentType", value = "外挂设备类型")
    private String equipmentType;
 
}