package com.huatek.busi.vo;

import java.io.Serializable;
import java.math.BigDecimal;
import java.time.LocalDateTime;

import org.springframework.format.annotation.DateTimeFormat;

import com.huatek.busi.common.annotation.ExcelAnnotation;

import lombok.Data;
import lombok.EqualsAndHashCode;

/**
 * 电容型设备——绝缘监测：022001 Vo
 *
 * @author anker_cheng
 * @date Aug 2, 2019 2:13:18 PM
 */
@Data
@EqualsAndHashCode(callSuper = false)
public class CapacitanceEquipmentImVo implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 主键
     */
    private String id;

    /**
     * CAC 的唯一标识（17 位码）
     */
    private String cacId;

    /**
     * 监测数据组的数目
     */
    private BigDecimal dataNodeNum;


    /**
     * 监测类型编码
     */
    private String cacMonitorTypeCode;

    /**
     * 在线监测装置对应的被监测设备的 唯一标识（17 位码）
     */
    private String equipmentId;

    
    /**
     * 设备编码
     */
    @ExcelAnnotation(exportName = "设备编号")
    private String systemCode;
    
    /**
     * 设备名称
     */
    @ExcelAnnotation(exportName = "设备名称")
    private String equipmentName;
    
    /**
     * 传感器编号
     */
    @ExcelAnnotation(exportName = "传感器编号")
    private String sensorId;
    
    /**
     * 传感器名称
     */
    @ExcelAnnotation(exportName = "传感器名称")
    private String sensorName;
    
    /**
     * 主设备编号
     */
    private String equipmentCode;

    /**
     * 外挂设备编号
     */
    private String extEquipmentCode;

    /**
     * 站点ID
     */
    private String siteId;


    /**
     * 数据采集时间，格式为：yyyy-MM-dd HH:mm:ss
     */
    @ExcelAnnotation(exportName = "监测时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime timeStamp;
    

    /**
     * 被监测设备相别
     */
    @ExcelAnnotation(exportName = "被监测设备相别")
    private String phase;

    /**
     * 泄露电流(mA)
     */
    @ExcelAnnotation(exportName = "泄露电流(mA)")
    private String totalCurrent;

    /**
     * 电容量(pF)
     */
    @ExcelAnnotation(exportName = "电容量(pF)")
    private String capacitance;

    /**
     * 介质损耗因数
     */
    @ExcelAnnotation(exportName = "介质损耗因数")
    private String lossFactor;

    /**
     * 三相不平衡电流
     */
    private String unbalanceCurrent;

    /**
     * 三相不平衡电压
     */
    private String unbalanceVoltage;


    /**
     * 系统电压(V)
     */
    @ExcelAnnotation(exportName = "系统电压(V)")
    private String systemVoltage;

    /**
     * 主设备ID
     */
    private String mainEquipmentId;

    /**
     * 外挂设备ID
     */
    private String subEquipmentId;

    /**
     * 站点设备树ID
     */
    private String mainTreeId;

    /**
     * 外挂设备树ID
     */
    private String subTreeId;

    private String type;
}