package com.huatek.busi.vo;

import java.io.Serializable;
import java.math.BigDecimal;
import java.time.LocalDateTime;

import org.springframework.format.annotation.DateTimeFormat;

import com.huatek.busi.common.annotation.ExcelAnnotation;
import com.huatek.torch.sql.annotation.FieldName;

import lombok.Data;
import lombok.EqualsAndHashCode;

/**
 * 断路器/GIS——SF6气体压力：024004、024005 Vo
 *
 * @author anker_cheng
 * @date Aug 2, 2019 2:13:18 PM
 */
@Data
@EqualsAndHashCode(callSuper = false)
public class GisSf6PressureVo implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 主键
     */
    private String id;

    /**
     * CAC 的唯一标识（17 位码）
     */
    private String cacId;

    /**
     * 监测数据组的数目
     */
    private BigDecimal dataNodeNum;

    /**
     * 监测类型编码
     */
    private String cacMonitorTypeCode;

    /**
     * 在线监测装置对应的被监测设备的 唯一标识（17 位码）
     */
    private String equipmentId;
    
   
    /**
     * 设备编码
     */
    @ExcelAnnotation(exportName = "设备编号")
    private String systemCode;

    
    /**
     * 设备名称
     */
    @ExcelAnnotation(exportName = "设备名称")
    private String equipmentName;
    
    /**
     * 在线监测装置的唯一标识（17 位码）
     */
    @ExcelAnnotation(exportName = "传感器编号")
    private String sensorId;
    
    /**
     * 在线监测装置的唯一标识（17 位码）
     */
    @ExcelAnnotation(exportName = "传感器名称")
    private String sensorName;
    /**
     * 数据采集时间，格式为：yyyy-MM-dd HH:mm:ss
     */
    @ExcelAnnotation(exportName = "监测时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime timeStamp;

    /**
     * 主设备编号
     */
    private String equipmentCode;

    /**
     * 外挂设备编号
     */
    private String extEquipmentCode;

    /**
     * 站点ID
     */
    private String siteId;

    /**
     * 被监测设备相别
     */
    @ExcelAnnotation(exportName = "设备相别")
    private String phase;

    /**
     * 温度（℃）
     */
    @ExcelAnnotation(exportName = "温度（℃）")
    private String temperature;

    /**
     * 绝对压力 (MPa)
     */
    private String absolutePressure;

    /**
     * 密度(kg/m3)
     */
    private String density;

    /**
     * 微水
     */
    @ExcelAnnotation(exportName = "水分(  μL/L )")
    private String moisture;

    /**
     * 压力（20℃）
     */
    @ExcelAnnotation(exportName = "压力（20℃）MPa")
    private String pressure20c;

    /**
     * 主设备ID
     */
    private String mainEquipmentId;

    /**
     * 外挂设备ID
     */
    private String subEquipmentId;

    /**
     * 站点设备树ID
     */
    private String mainTreeId;

    /**
     * 外挂设备树ID
     */
    private String subTreeId;
    
    /**
     * 折合到20度的压力值
     */
    @FieldName("pressure")
    private String pressure;
}