package com.huatek.busi.vo;

import java.io.Serializable;
import java.math.BigDecimal;
import java.time.LocalDateTime;

import org.springframework.format.annotation.DateTimeFormat;

import com.huatek.busi.common.annotation.ExcelAnnotation;

import lombok.Data;
import lombok.EqualsAndHashCode;

/**
 * 变压器/电抗器——气象：021011、023003、024008、025001 Vo
 *
 * @author anker_cheng
 * @date Aug 2, 2019 2:13:18 PM
 */
@Data
@EqualsAndHashCode(callSuper = false)
public class TransformerMeteorologicalVo implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 主键
     */
    private String id;

    /**
     * CAC 的唯一标识（17 位码）
     */
    private String cacId;

    /**
     * 监测数据组的数目
     */
    private BigDecimal dataNodeNum;

    /**
     * 在线监测装置的唯一标识（17 位码）
     */
    private String sensorId;

    /**
     * 监测类型编码
     */
    private String cacMonitorTypeCode;

    /**
     * 在线监测装置对应的被监测设备的 唯一标识（17 位码）
     */
    private String equipmentId;
    
    /**
     * 设备编码
     */
    @ExcelAnnotation(exportName = "设备编号")
    private String systemCode;
    
    /**
     * 设备名称
     */
    @ExcelAnnotation(exportName = "设备名称")
    private String equipmentName;
    
    
    /**
     * 传感器编号
     */
    @ExcelAnnotation(exportName = "传感器编号")
    private String sensorCode;
    
    /**
     * 传感器名称
     */
    @ExcelAnnotation(exportName = "传感器名称")
    private String sensorName;

    /**
     * 数据采集时间，格式为：yyyy-MM-dd HH:mm:ss
     */
    @ExcelAnnotation(exportName = "监测时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime timeStamp;

    /**
     * 主设备编号
     */
    private String equipmentCode;

    /**
     * 外挂设备编号
     */
    private String extEquipmentCode;

    /**
     * 站点ID
     */
    private String siteId;

    /**
     * 被监测设备相别
     */
    private String phase;

    /**
     * 气温(℃)
     */
    @ExcelAnnotation(exportName = "气温(℃)")
    private String airTemperature;

    /**
     * 气压
     */
    @ExcelAnnotation(exportName = "气压")
    private String airPressure;

    /**
     * 湿度
     */
    @ExcelAnnotation(exportName = "湿度")
    private String humidity;

    /**
     * 降雨量
     */
    @ExcelAnnotation(exportName = "降雨量")
    private String precipitation;

    /**
     * 降水强度
     */
    @ExcelAnnotation(exportName = "降水强度")
    private String precipitationIntensity;

    /**
     * 光辐射强度
     */
    @ExcelAnnotation(exportName = "光辐射强度")
    private String radiationIntensity;

    /**
     * 主设备ID
     */
    private String mainEquipmentId;

    /**
     * 外挂设备ID
     */
    private String subEquipmentId;

    /**
     * 站点设备树ID
     */
    private String mainTreeId;

    /**
     * 外挂设备树ID
     */
    private String subTreeId;
}