package com.huatek.busi.service;

import com.huatek.busi.vo.BusiInfraredVo;
import com.huatek.busi.vo.BusiWindingVo;
import com.huatek.busi.vo.cable.BusiCableCirculationWarningVo;
import com.huatek.busi.vo.warning.BusiArresterVo;
import com.huatek.busi.vo.warning.BusiCapaVo;
import com.huatek.busi.vo.warning.BusiGisSf6Vo;
import com.huatek.busi.vo.warning.BusiIcgeVo;
import com.huatek.busi.vo.warning.BusiOilVo;
import com.huatek.busi.vo.warning.BusiPdVo;
import com.huatek.busi.vo.warning.BusiPhotoVo;
import com.huatek.busi.vo.warning.BusiRunStateVo;
import com.huatek.busi.vo.warning.BusiWcgVo;

/**
 * @ClassName: BusiSubPageService
 * @FullClassPath: com.huatek.busi.service.BusiSubPageService
 * @Description: 二级页面服务.
 * @author: Arno
 * @date: Aug 2, 2019 3:05:55 PM
 * @version: 1.0
 */

public interface BusiSubPageService {

	/**
	 * @Title: getOilWarnigInfo
	 * @Description: 铁芯夹件告警信息
	 * @createDate: Aug 2, 2019 3:08:37 PM
	 * @param    warningId    告警ID
	 * @param    sensorCode   传感器编码
	 * @return   BusiIcgeVo
	 */
	BusiIcgeVo getIcgeWarningInfo(String warningId, String sensorCode);


	/**
	 * @Title: getOilWarnigInfo
	 * @Description: 油色谱告警信息
	 * @createDate: Aug 2, 2019 3:08:37 PM
	 * @param    warningId    告警ID
	 * @param    sensorCode   传感器编码
	 * @return   BusiOilVo
	 */
	BusiOilVo getOilWarnigInfo(String warningId, String sensorCode);

	/**
	 * 避雷器告警信息
	 *
	 * @param warningId
	 *            告警ID
	 * @param sensorCode
	 *            传感器编码
	 * @return BusiArresterVo
	 */
	BusiArresterVo getArresterInfo(String warningId, String sensorCode);

	/**
	 * 容性设备告警信息
	 *
	 * @param warningId
	 *            告警ID
	 * @param sensorCode
	 *            传感器编码
	 * @return BusiArresterVo
	 */
	BusiCapaVo getCapacitanceInfo(String warningId, String sensorCode);

	/**
	 * @Title: getPdWarningInfo
	 * @Description: 局放告警信息
	 * @createDate: Aug 2, 2019 3:08:37 PM
	 * @param    warningId    告警ID
	 * @param    sensorCode   传感器编码
	 * @param    pdType       局放类型
	 * @return   BusiPdVo
	 */
	BusiPdVo getPdWarningInfo(String warningId, String sensorCode, String pdType);

	/**
	 * @Title: getcableCirculationWarningInfo
	 * @Description: 局放告警信息
	 * @createDate: Aug 2, 2019 3:08:37 PM
	 * @param    warningId    告警ID
	 * @param    sensorCode   传感器编码
	 * @param    eqType       设备类型
	 * @return   BusiPdVo
	 */
	BusiCableCirculationWarningVo getcableCirculationWarningInfo(String warningId, String sensorCode, String eqType);

	/**
	 * @Title: getPhotoWarningInfo
	 * @Description: 图片告警信息
	 * @createDate: Aug 2, 2019 3:08:37 PM
	 * @param    warningId    告警ID
	 * @param    sensorCode   传感器编码
	 * @param    pdType       局放类型
	 * @return   BusiPhotoVo
	 */
	BusiPhotoVo getPhotoWarningInfo(String warningId, String sensorCode, String pdType);


	/**
	 * Gis-SF6告警信息
	 *
	 * @param warningId
	 *            告警ID
	 * @param sensorCode
	 *            传感器编码
	 * @return BusiGisSf6Vo
	 */
	BusiGisSf6Vo getGisSf6Info(String warningId, String sensorCode);

	/**
	 * 红外检测告警
	 * @param warningId
	 * @param sensorCode
	 * @return
	 */
	BusiInfraredVo getBusiInfrared(String warningId, String sensorCode,String pdType);

	/**
	 * 绕组测温告警
	 * @param warningId
	 * @param sensorCode
	 * @return
	 */
	BusiWindingVo getBusiWinding(String warningId, String sensorCode, String pdType);

	/**
	 * 运行工况告警信息
	 *
	 * @param warningId
	 *            告警ID
	 * @param mainTreeId
	 *            站点设备树ID
	 * @return BusiGisSf6Vo
	 */
	BusiRunStateVo getRunStateWarningInfo(String warningId, String mainTreeId);

	/**
	 * @Title: getCwgInfo
	 * @Description:测温柜告警信息
	 * @createDate: Aug 2, 2019 3:08:37 PM
	 * @param    warningId    告警ID
	 * @param    sensorCode   传感器编码
	 * @return   BusiWcgVo
	 */
	BusiWcgVo getCwgInfo(String warningId, String sensorCode);
}
