package com.huatek.busi.mapper;

import java.util.List;

import com.huatek.busi.entity.BusiParamTransformer;
import com.huatek.busi.entity.BusiParamTransformerCalculate;
import com.huatek.busi.entity.BusiParamTransformerVoluntarily;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import com.huatek.busi.vo.BusiParamTransformerVo;

/**
 * 变压器的权重系数维护表 Mapper
 *
 * @author anker_cheng
 * @date Jul 12, 2019 2:57:22 PM
 */
@Repository
public interface BusiParamTransformerMapper {
    /**
     * 获取一级设备分析参数列表
     * 
     * @param siteId
     *            站点ID
     * @param mainEquipmentId
     *            主设备ID
     * @return 分析参数列表
     */
    List<BusiParamTransformerVo> findFirstLevelParams(@Param("siteId") String siteId,
        @Param("mainEquipmentId") String mainEquipmentId,@Param("number") Integer number);

    /**
     * 获取一级设备分析参数列表(无值)
     *
     * @param siteId
     *            站点ID
     * @param mainEquipmentId
     *            主设备ID
     * @return 分析参数列表
     */
    List<BusiParamTransformerVo> findNotValueFirstLevelParams(@Param("siteId") String siteId,
                                                      @Param("mainEquipmentId") String mainEquipmentId,
                                                              @Param("number") Integer number);

    /**
     * 获取分析参数的子分类列表
     * 
     * @param siteId
     *            站点ID
     * @param mainEquipmentId
     *            主设备ID
     * @param pname
     *            上级参数名称
     * @return 分析参数列表
     */
    List<BusiParamTransformerVo> findSubParams(@Param("siteId") String siteId,
        @Param("mainEquipmentId") String mainEquipmentId, @Param("pname") String pname,@Param("number") Integer number);
    
    List<BusiParamTransformerVo> findParams(@Param("siteId") String siteId,
            @Param("mainEquipmentId") String mainEquipmentId);

    List<BusiParamTransformerVoluntarily> findTransformerParma(@Param("id") String id, @Param("siteId") String siteId, @Param("mainEquipmentId") String mainEquipmentId);

    List<BusiParamTransformerVoluntarily> findVoluntarilyParam(@Param("siteId") String siteId, @Param("mainEquipmentId") String mainEquipmentId);

    List<BusiParamTransformerVo> findParamsManual(@Param("siteId") String siteId, @Param("mainEquipmentId") String mainEquipmentId);

    List<BusiParamTransformerVoluntarily> findSubParamsVo( @Param("siteId") String siteId,@Param("mainEquipmentId") String mainEquipmentId,@Param("name") String name);

    List<BusiParamTransformerVo> findSubParamsCalculate(@Param("siteId") String siteId,
                                                        @Param("mainEquipmentId") String mainEquipmentId, @Param("pname") String pname);

    List<BusiParamTransformerCalculate> findTransformerParmaCalculate(@Param("id") String id, @Param("siteId") String siteId, @Param("mainEquipmentId") String mainEquipmentId);

    BusiParamTransformerVo findFirstLevelNumber(@Param("siteId") String siteId,
    @Param("mainEquipmentId") String mainEquipmentId);
}
