package com.huatek.busi.task.transformer.part;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import com.huatek.busi.common.SiteConstants.SiteTreeEnum;
import com.huatek.busi.common.SiteConstants.TransfromerTreeEnum;
import com.huatek.busi.common.UploadCycle.ConfigField;
import com.huatek.busi.service.BusiSensorService;
import com.huatek.busi.service.FailureRateService;
import com.huatek.busi.service.SiteUploadCycleService;
import com.huatek.busi.service.transformer.TransformerReadTableService;
import com.huatek.busi.task.base.AbstractBusiTaskExecutor;
import com.huatek.busi.vo.BusiSensorVo;
import com.huatek.busi.vo.ReadTableCountVo;

/**
 * 变压器读表统计
 * 
 * @author anker_cheng
 * @date 2019/10/28
 */
@Component("transfromReadTask")
public class TransfromReadTask extends AbstractBusiTaskExecutor {
    @Autowired
    private BusiSensorService busiSensorService;
    @Autowired
    private TransformerReadTableService transformerReadTableService;
    @Autowired
    private SiteUploadCycleService siteUploadCycleService;
    @Autowired
    private FailureRateService failureRateService;

    @Override
    public void execute(LocalDate day) {
        List<BusiSensorVo> sensorList = findSensorList(TransfromerTreeEnum.TP_EQM.getValue());
        if (CollectionUtils.isEmpty(sensorList)) {
            return;
        }
        LocalDateTime startTime = day.atStartOfDay();
        LocalDateTime endTime = day.plusDays(1).atStartOfDay();
        for (BusiSensorVo sensor : sensorList) {
            // 上传周期
            BigDecimal cycle = siteUploadCycleService.getCycle(sensor.getSiteId(), ConfigField.pzzz);
            // 数据统计
            ReadTableCountVo item =
                transformerReadTableService.getListEquInfo(sensor.getSiteId(), sensor.getSensorCode(), startTime, endTime);
            // 保存缺失率
            failureRateService.saveSensorFailureRate(sensor, day, cycle, item == null ? 0 : item.getDataCount());
        }
    }

    private List<BusiSensorVo> findSensorList(String equipmentType) {
        return busiSensorService.findSensorList(SiteTreeEnum.BYQ, new String[] {equipmentType});
    }
}
