/***************GIS开关评测start***************/
/**
 * GIS 打分
 * @param yxsj 运行数据
 * @param jyjz 绝缘介质
 * @param kdms 开断磨损数据
 * @param yxcs 运行参数
 * @param a 运行数据（0.106）
 * @param b 绝缘介质（0.189）
 * @param c 开断磨损数据（0.599）
 * @param d 运行参数（0.106）
 * @returns {Number}
 */
function gis_score(yxsj, jyjz, kdms,yxcs , a, b, c, d) {
	
	return (yxsj*a+jyjz*b+kdms*c+yxcs*d)*100;
}

/******运行数据start*****/
/**
 * 运行数据得分
 * @param dqsy 运行数据-电气试验
 * @param sjcs 时间参数
 * @param sdtx 速度参数
 * @param a 运行数据-电气试验权重（0.157）
 * @param b 时间参数权重（0.594）
 * @param c 速度特性权重（0.249）
 * @returns {Number}
 */
function gis_yxsj(dqsy, sjcs, sdtx, a, b, c) {
	
	return dqsy*a+sjcs*b+sdtx*c;
}
/**
 * 运行数据-电气试验
 * @param ddhldz 导电回路电阻
 * @param a 导电回路电阻权重（0.157）
 * @returns {Number}
 */
function gis_yxsj_ddhldz(ddhldz, a) {
	
	return ddhldz*a;
}
/**
 * 运行数据——时间参数得分
 * @param hzbtq 合闸不同期
 * @param fzbtq 分闸不同期
 * @param a 合闸不同期权重（0.323）
 * @param b 分闸不同期权重（0.588）
 * @returns {Number}
 */
function gis_yxsj_sjcs(hzbtq, fzbtq, a, b) {
	
	return hzbtq*a+fzbtq*b;
}
/**
 * 运行数据——速度参数得分
 * @param gfsd 刚分速度
 * @param ghsd 刚和速度
 * @param a 刚分速度权重（0.323）
 * @param b 刚和速度权重（0.588）
 * @returns {Number}
 */
function gis_yxsj_sdtx(gfsd, ghsd, a, b) {
	
	return gfsd*a+ghsd*b;
}

/******运数据end*****/
/******绝缘介质start*****/
/**
 * 绝缘介质（0.189）-SF6在线数据
 * @param qtyl 气体压力
 * @param a 气体压力权重（0.310）
 * @param wshl 微水含量
 * @param b 微水含量权重（0.581）
 * @param jfl 局放量
 * @param d 局放量权重
 * @param jfcs 局放次数
 * @param e 局放次数权重
 * @returns {Number}
 */
function gis_jyjz(qtyl, a, wshl ,b,jfl, d,jfcs,e) {
	
	return qtyl*a+wshl*b+jfl*d+jfcs*e;
}
/******绝缘介质end*****/

/******开断磨损start*****/
/**
 * 开断磨损数据（0.599）
 * @param ljdzcs 累计动作次数
 * @param syns 使用年数
 * @param xddmscd 相对电磨损程度

 * @param a 累计动作次数权重（0.112）
 * @param b 使用年数权重（0.130）
 * @param c 相对电磨损程度权重（0.758）
 * @returns {Number}
 */
function gis_kdms(ljdzcs, syns, xddmscd, a, b, c) {
	
	return ljdzcs*a+syns*b+xddmscd*c;
}

/**
 * 开断磨损（0.599）-在线局放数据
 * @param jfl 局放量
 * @param jfcs 局放次数
 * @param a 局放量权重（0.6）
 * @param b 局放次数权重（0.4）
 * @returns {Number}
 */
function gis_jfsj(jfl, jfcs, a, b) {
	
	return jfl*a+jfcs*b;
}

/******开断磨损end*****/

/******运行参数start*****/

/**
 * 运行参数
 * @param gzhj 工作环境
 * @param a 工作环境权重（0.09）
 * @param wgqk 外观情况
 * @param b 外观情况权重（0.364）
 * @param sccj 生产厂家
 * @param c 生产厂家权重（0.204）
 * @param dlqwh 断路器维护
 * @param d 断路器维护权重（0.204）
 * @returns {Number}
 */
function gis_yxcs(gzhj, a, wgqk, b, sccj ,c,dlqwh ,d) {
	
	return gzhj*a+wgqk*b+sccj*c+dlqwh*d;
}


/**
 * 运行参数-工作环境
 * @param wd 使用环境温度
 * @param a 使用环境温度权重（0.495）
 * @param sd 使用环境湿度
 * @param b 使用环境湿度权重（0.267）
 * @param fsqtjca 腐蚀气体及尘埃
 * @param c 腐蚀气体及尘埃权重（0.064）
 * @param qtys 其他因素
 * @param d 其他因素权重（0.164）
 * @returns {Number}
 */
function gis_yxcs_gzhj(wd, a, sd ,b ,fsqtjca ,c, qtys, d) {
	
	return wd*a+sd*b+fsqtjca*c+qtys*d;
}
/**
 * 运行参数-外观情况
 * @param czjgrh 操作机构润滑(检修记录)
 * @param a 操作机构润滑（0.273）
 * @param ztjgwg 主体、机构外观(检修记录)
 * @param b 主体、机构外观（0.177）
 * @param zhldz 主回路端子
 * @param c 主回路端子权重（0.088）
 * @param zcjybf 支持绝缘部分
 * @param d 支持绝缘部分权重（0.462）
 * @param tldlqwhcs 同类断路器维护次数(检修记录)
 * @param e 同类断路器维护次数权重（0.462）
 * @returns {Number}
 */
function gis_yxcs_wgqk(czjgrh, a, ztjgwg ,b ,zhldz ,c ,zcjybf ,d,tldlqwhcs ,e) {
	
	return czjgrh*a+ztjgwg*b+zhldz*c+zcjybf*d+tldlqwhcs*e;
}

/**
 * 运行参数-生产厂家
 * @param cjsbyc 厂家设备遗传（1）
 * @param a 厂家设备遗传权重（0.297）
 * @param cprzd 产品认知度
 * @param b 产品认知度权重（0.539）
 * @param cpcsd 产品成熟度
 * @param c 产品成熟度权重（0.164）
 * @returns {Number}
 */
function gis_yxcs_sccj(cjsbyc, a, cprzd ,b ,cpcsd ,c) {
	
	return cjsbyc*a+cprzd*b+cpcsd*c;
}

/******运行参数end*****/
/***************GIS开关评测end***************/